<?php
	
class DAP_UserGroupAccount {
	
   	var $id;
   	var $userid;
   	var $productid;
   	var $seats;
   	var $total_created;
   	var $purchase_date; 
	
	function getId() {
		return $this->id;
	}
	function setId($o) {
		$this->id = $o;
	}
	function getUserId() {
		return $this->userid;
	}
	function setUserId($o) {
		$this->userid = $o;
	}
	
	function getProductId() {
		return $this->productid;
	}
	function setProductId($o) {
		$this->productid = $o;
	}

	function getSeats() {
		return $this->seats;
	}
	function setSeats($o) {
		$this->seats = $o;
	}
	function getTotalCreated() {
		return $this->total_created;
	}
	function setTotalCreated($o) {
		$this->total_created = $o;
	}
	 
	function getPurchaseDate() {
		return $this->purchase_date;
	}
	function setPurchaseDate($o) {
		$this->purchase_date = $o;
	}
 	
	public function create() {
		
		try {
			$dap_dbh = Dap_Connection::getConnection();
			$sql = "insert into dap_user_group_account
						( userid, productid, seats, total_created, purchase_date)
					values 
						(:userid,  :productid, :seats, :total_created, now())";

			$stmt = $dap_dbh->prepare($sql);
				 
 
			$stmt->bindParam(':userid', $this->getUserId(), PDO::PARAM_INT);
			$stmt->bindParam(':productid', $this->getProductId(), PDO::PARAM_INT);
			$stmt->bindParam(':seats', $this->getSeats(), PDO::PARAM_INT);
			$stmt->bindParam(':total_created', $this->getTotalCreated(), PDO::PARAM_STR);
			$stmt->execute();
			 
			$last_insert_id = $dap_dbh->lastInsertId();
	 
			$stmt = null;
			$dap_dbh = null;
			return $last_insert_id;
	
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}	
	}
 
	public function update() {
		try {
			$dap_dbh = Dap_Connection::getConnection();
	
			$sql = "update 
					dap_user_group_account
					set
						 userid =:userid, 
						 productid =:productid, 
						 seats=:seats,
						 total_created=:total_created,
						 purchase_date=now() 
					where
						id =:id ";
				 
			$stmt = $dap_dbh->prepare($sql);	
			
			$stmt->bindParam(':userid', $this->getUserId(), PDO::PARAM_INT);
			$stmt->bindParam(':productid', $this->getProductId(), PDO::PARAM_INT);
			$stmt->bindParam(':seats', $this->getSeats(), PDO::PARAM_INT);
			$stmt->bindParam(':total_created', $this->getTotalCreated(), PDO::PARAM_STR);
			$stmt->bindParam(':id', $this->getId(), PDO::PARAM_INT);
			$stmt->execute();
			
			$stmt = null;
			$dap_dbh = null;
			
			return;
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}	
	}
	
	
	
	public function updateUserSeats() {
		try {
			$dap_dbh = Dap_Connection::getConnection();
	
			$sql = "update 
					dap_user_group_account
					set
						 seats=:seats,
						 total_created=:total_created,
						 purchase_date=now() 
					where
						productid =:productid  and userid=:userid";
				 
			$stmt = $dap_dbh->prepare($sql);	
			
			$stmt->bindParam(':userid', $this->getUserId(), PDO::PARAM_INT);
			$stmt->bindParam(':productid', $this->getProductId(), PDO::PARAM_INT);
			$stmt->bindParam(':seats', $this->getSeats(), PDO::PARAM_INT);
			$stmt->bindParam(':total_created', $this->getTotalCreated(), PDO::PARAM_INT);
			$stmt->execute();
			
			$stmt = null;
			$dap_dbh = null;
			
			return;
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}	
	}
	
	
	public function updateUserTotalSeats() {
		try {
			$dap_dbh = Dap_Connection::getConnection();
	
			$sql = "update 
					dap_user_group_account
					set
						 seats=:seats
					where
						productid =:productid  and userid=:userid";
				 
			$stmt = $dap_dbh->prepare($sql);	
			
			$stmt->bindParam(':userid', $this->getUserId(), PDO::PARAM_INT);
			$stmt->bindParam(':productid', $this->getProductId(), PDO::PARAM_INT);
			$stmt->bindParam(':seats', $this->getSeats(), PDO::PARAM_INT);
			$stmt->execute();
			
			$stmt = null;
			$dap_dbh = null;
			
			return;
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}	
	}
	
	
	
	public static function load() {
		$dataarr = array();
		
		try {
			$dap_dbh = Dap_Connection::getConnection();
			
			$sql = "SELECT 
						*
					FROM
						dap_user_group_account ORDER BY date DESC
					";
			//logToFile($sql,LOG_DEBUG_DAP);
			$stmt = $dap_dbh->prepare($sql);
			 
			$stmt->execute();
	
			while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
				$data = new DAP_UserGroupAccount();
 
				$data->setId( stripslashes($row["id"]) );
				$data->setUserId( stripslashes($row["userid"]) );
				$data->setProductId( stripslashes($row["productid"]) );
				$data->setSeats( stripslashes($row["seats"]) );
				$data->setTotalCreated( stripslashes($row["total_created"]) );
				$data->setPurchaseDate( stripslashes($row["purchase_date"]) ); 
				$dataarr[] = $data;

			}
			
			
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
		
		return $dataarr;
	}
	
	 public static function loadByUserID($userid) {
		$dataarr = array();
		
		try {
			$dap_dbh = Dap_Connection::getConnection();
			
			$sql = "SELECT 
						*
					FROM
						dap_user_group_account
					WHERE
						userid =:userid 
						
					";
			// logToFile("sql=".$sql,LOG_DEBUG_DAP);
			$stmt = $dap_dbh->prepare($sql);
			$stmt->bindParam(':userid', $userid, PDO::PARAM_STR);
			$stmt->execute();
	         
			while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {				 
				$dataarr[] = $row;
			}	
			$stmt = null;
			$dap_dbh = null;		
			
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
		
		return $dataarr;
	} 
	
	 
	
	public static function loadByName($name) {
		$dataarr = array();
		
		try {
			$dap_dbh = Dap_Connection::getConnection();
			
			$sql = "SELECT 
						*
					FROM
						dap_user_group_account
					WHERE
						name =:name 
						
					";
			// logToFile("sql=".$sql,LOG_DEBUG_DAP);
			$stmt = $dap_dbh->prepare($sql);
			$stmt->bindParam(':name', $name, PDO::PARAM_STR);
			$stmt->execute();
	         
			while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
				$data = new DAP_UserGroupAccount();
				
				$data->setId( stripslashes($row["id"]) );
				$data->setUserId( stripslashes($row["userid"]) );
				$data->setProductId( stripslashes($row["productid"]) );
				$data->setSeats( stripslashes($row["seats"]) );
				$data->setTotalCreated( stripslashes($row["total_created"]) );
				$data->setPurchaseDate( stripslashes($row["purchase_date"]) );  
				$dataarr[] = $data;

			}			
			
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
		
		return $dataarr;
	} 
 
	public function delete($id) {
		try {
			$dap_dbh = Dap_Connection::getConnection();
			$dap_dbh->beginTransaction();
			
			//$id = $this->getId();
			//delete from usersproducts table
			  $sql = "delete from  
					dap_user_group_account
					where id =:id"; 
			$stmt = $dap_dbh->prepare($sql);
			$stmt->bindParam(':id', $id, PDO::PARAM_INT);
			$stmt->execute();
			
			$dap_dbh->commit(); 
			
			$stmt = null;
			$dap_dbh = null;		
			return;			
		} catch (PDOException $e) {
			$dap_dbh->rollback();
			$stmt = null;
			$dap_dbh = null;				
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			$dap_dbh->rollback();
			$stmt = null;
			$dap_dbh = null;			
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}		
	}
	
	public static function loadById($id) {
		$dataarr = null;
		
		try {
			$dap_dbh = Dap_Connection::getConnection();
			
			$sql = "SELECT 
						*
					FROM
						dap_user_group_account
					WHERE
						id =:id 
						
					";
			// logToFile("sql=".$sql,LOG_DEBUG_DAP);
			$stmt = $dap_dbh->prepare($sql);
			$stmt->bindParam(':id', $id, PDO::PARAM_STR);
			$stmt->execute();
	        if ($row = $stmt->fetch()) {
				$data = new DAP_UserGroupAccount();
				$data->setId( stripslashes($row["id"]) );
				$data->setUserId( stripslashes($row["userid"]) );
				$data->setProductId( stripslashes($row["productid"]) );
				$data->setSeats( stripslashes($row["seats"]) );
				$data->setTotalCreated( stripslashes($row["total_created"]) );
				$data->setPurchaseDate( stripslashes($row["purchase_date"]) ); 
				$dataarr = $data;

			}
			$stmt = null;
			$dap_dbh = null;
			return $dataarr;				
			
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
		
		return $dataarr;
	} 
	
	public static function loadByUserAndProductID($userid, $productid) {
		$dataarr = array();
		
		try {
			$dap_dbh = Dap_Connection::getConnection();
			
			$sql = "SELECT 
						*
					FROM
						dap_user_group_account
					WHERE
						userid =:userid and
						productid =:productid 
						
					";
			// logToFile("sql=".$sql,LOG_DEBUG_DAP);
			$stmt = $dap_dbh->prepare($sql);
			$stmt->bindParam(':userid', $userid, PDO::PARAM_STR);
			$stmt->bindParam(':productid', $productid, PDO::PARAM_STR);
			$stmt->execute();
	         
			while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {				 
				$dataarr[] = $row;
			}	
			$stmt = null;
			$dap_dbh = null;		
			
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
		
		return $dataarr;
	} 
	
	
	public static function loadByProductId($productId) {
		$dataarr = array();
		
		try {
			$dap_dbh = Dap_Connection::getConnection();
			
			$sql = "SELECT 
						*
					FROM
						dap_user_group_account 
					WHERE
						productid =:productid
					";
			//logToFile($sql,LOG_DEBUG_DAP);
			$stmt = $dap_dbh->prepare($sql);
			$stmt->bindParam(':productid', $productId, PDO::PARAM_STR);
			 
			$stmt->execute();
	
			while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
				$data = new DAP_UserGroupAccount();
 
				$data->setId( stripslashes($row["id"]) );
				$data->setUserId( stripslashes($row["userid"]) );
				$data->setProductId( stripslashes($row["productid"]) );
				$data->setSeats( stripslashes($row["seats"]) );
				$data->setTotalCreated( stripslashes($row["total_created"]) );
				$data->setPurchaseDate( stripslashes($row["purchase_date"]) ); 
				$dataarr[] = $data;

			}
			
			
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
		
		return $dataarr;
	}
	
 
}
 
